@@ -19,18 +19,11 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-'use strict';
-
-/******************************************************************************/
-
-import contextMenu from './contextmenu.js';
-import logger from './logger.js';
-import scriptletFilteringEngine from './scriptlet-filtering.js';
-import staticNetFilteringEngine from './static-net-filtering.js';
-import µb from './background.js';
-import webext from './webext.js';
-import { PageStore } from './pagestore.js';
-import { i18n$ } from './i18n.js';
+import {
+    domainFromHostname,
+    hostnameFromURI,
+    originFromURI,
+} from './uri-utils.js';
 
 import {
     sessionFirewall,
@@ -38,11 +31,14 @@ import {
     sessionURLFiltering,
 } from './filtering-engines.js';
 
-import {
-    domainFromHostname,
-    hostnameFromURI,
-    originFromURI,
-} from './uri-utils.js';
+import { PageStore } from './pagestore.js';
+import contextMenu from './contextmenu.js';
+import { i18n$ } from './i18n.js';
+import logger from './logger.js';
+import scriptletFilteringEngine from './scriptlet-filtering.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import webext from './webext.js';
+import µb from './background.js';
 
 /******************************************************************************/
 /******************************************************************************/
@@ -65,7 +61,7 @@ import {
         }
         try {
             tabURLNormalizer.href = tabURL;
-        } catch(ex) {
+        } catch {
             return tabURL;
         }
         const protocol = tabURLNormalizer.protocol.slice(0, -1);
@@ -574,18 +570,15 @@ housekeep itself.
                         frameId: sourceFrameId,
                     }),
                 ]);
-            }
-            catch (reason) {
-                return;
-            }
-            if (
-                Array.isArray(openerDetails) === false ||
-                openerDetails.length !== 2 ||
-                openerDetails[1] === null ||
-                openerDetails[1].url === 'about:newtab'
-            ) {
+            } catch {
                 return;
             }
+            if ( Array.isArray(openerDetails) === false ) { return; }
+            if ( openerDetails.length !== 2 ) { return; }
+            if ( openerDetails[1] === null ) { return; }
+            if ( openerDetails[1].url === 'about:newtab' ) { return; }
+            // https://github.com/uBlockOrigin/uBlock-issues/issues/2227
+            if ( openerDetails[1].url.startsWith('chrome:') ) { return; }
             popupCandidates.set(
                 tabId,
                 new PopupCandidate(createDetails, openerDetails)
@@ -933,6 +926,7 @@ vAPI.Tabs = class extends vAPI.Tabs {
         if ( pageStore === null ) { return; }
         pageStore.setFrameURL(details);
         if ( pageStore.getNetFilteringSwitch() ) {
+            details.ancestors = pageStore.getFrameAncestorDetails(frameId);
             scriptletFilteringEngine.injectNow(details);
         }
     }
